@@ -1,4 +1,3 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2009-2011 Collabora Ltd.
  *
@@ -135,7 +134,6 @@ pendingproc_prepare_tp_text_channel (TplActionChain *ctx,
       0
   };
 
-  /* user_data is a TplChannel instance */
   tp_proxy_prepare_async (chan, chan_features, channel_prepared_cb, ctx);
 }
 
@@ -496,7 +494,7 @@ on_pending_message_removed_cb (TpTextChannel *self,
   GList *ids = NULL;
   GError *error = NULL;
 
-  ids = g_list_append (ids,
+  ids = g_list_prepend (ids,
       GUINT_TO_POINTER (get_message_pending_id (TP_MESSAGE (message))));
 
   cache = _tpl_log_store_sqlite_dup ();
@@ -515,7 +513,8 @@ on_pending_message_removed_cb (TpTextChannel *self,
 
 
 static gint
-pending_message_compare (TpSignalledMessage *m1, TpSignalledMessage *m2)
+pending_message_compare_id (TpSignalledMessage *m1,
+    TpSignalledMessage *m2)
 {
   guint id1, id2;
 
@@ -531,6 +530,24 @@ pending_message_compare (TpSignalledMessage *m1, TpSignalledMessage *m2)
 }
 
 
+static gint
+pending_message_compare_timestamp (TpSignalledMessage *m1,
+    TpSignalledMessage *m2)
+{
+  gint64 ts1, ts2;
+
+  ts1 = get_message_timestamp (TP_MESSAGE (m1));
+  ts2 = get_message_timestamp (TP_MESSAGE (m2));
+
+  if (ts1 > ts2)
+    return 1;
+  else if (ts1 < ts2)
+    return -1;
+  else
+    return 0;
+}
+
+
 static void
 pendingproc_store_pending_messages (TplActionChain *ctx,
     gpointer user_data)
@@ -559,7 +576,7 @@ pendingproc_store_pending_messages (TplActionChain *ctx,
     tp_text_channel_get_pending_messages (TP_TEXT_CHANNEL (self));
 
   pending_messages = g_list_sort (pending_messages,
-      (GCompareFunc) pending_message_compare);
+      (GCompareFunc) pending_message_compare_id);
 
   cached_it = cached_messages;
   pending_it = pending_messages;
@@ -574,7 +591,7 @@ pendingproc_store_pending_messages (TplActionChain *ctx,
       if (cached_it == NULL)
         {
           /* No more cached pending, just log the pending messages */
-          to_log = g_list_append (to_log, pending_it->data);
+          to_log = g_list_prepend (to_log, pending_it->data);
           pending_it = g_list_next (pending_it);
           continue;
         }
@@ -584,7 +601,7 @@ pendingproc_store_pending_messages (TplActionChain *ctx,
       if (pending_it == NULL)
         {
           /* No more pending, just remove the cached messages */
-          to_remove = g_list_append (to_remove, GUINT_TO_POINTER (cached->id));
+          to_remove = g_list_prepend (to_remove, GUINT_TO_POINTER (cached->id));
           cached_it = g_list_next (cached_it);
           continue;
         }
@@ -598,7 +615,7 @@ pendingproc_store_pending_messages (TplActionChain *ctx,
           if (cached->timestamp != pending_ts)
             {
               /* The cache messaged is invalid, remove it */
-              to_remove = g_list_append (to_remove,
+              to_remove = g_list_prepend (to_remove,
                   GUINT_TO_POINTER (cached->id));
               cached_it = g_list_next (cached_it);
             }
@@ -612,13 +629,13 @@ pendingproc_store_pending_messages (TplActionChain *ctx,
       else if (cached->id < pending_id)
         {
           /* The cached ID is not valid anymore, remove it */
-          to_remove = g_list_append (to_remove, GUINT_TO_POINTER (cached->id));
+          to_remove = g_list_prepend (to_remove, GUINT_TO_POINTER (cached->id));
           cached_it = g_list_next (cached_it);
         }
-      else if (cached->id < pending_id)
+      else
         {
           /* The pending message has not been logged */
-          to_log = g_list_append (to_log, pending);
+          to_log = g_list_prepend (to_log, pending);
           pending_it = g_list_next (pending_it);
         }
     }
@@ -644,6 +661,9 @@ pendingproc_store_pending_messages (TplActionChain *ctx,
     {
       GList *it;
 
+      to_log = g_list_sort (to_log,
+          (GCompareFunc) pending_message_compare_timestamp);
+
       for (it = to_log; it != NULL; it = g_list_next (it))
         on_message_received_cb (TP_TEXT_CHANNEL (self),
             TP_SIGNALLED_MESSAGE (it->data), self);
